//C-  -*- C++ -*-
//C- -------------------------------------------------------------------
//C- DjVuLibre-3.5
//C- Copyright (c) 2002  Leon Bottou and Yann Le Cun.
//C- Copyright (c) 2001  AT&T
//C-
//C- This software is subject to, and may be distributed under, the
//C- GNU General Public License, either Version 2 of the license,
//C- or (at your option) any later version. The license should have
//C- accompanied the software or you may obtain a copy of the license
//C- from the Free Software Foundation at http://www.fsf.org .
//C-
//C- This program is distributed in the hope that it will be useful,
//C- but WITHOUT ANY WARRANTY; without even the implied warranty of
//C- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//C- GNU General Public License for more details.
//C- 
//C- DjVuLibre-3.5 is derived from the DjVu(r) Reference Library from
//C- Lizardtech Software.  Lizardtech Software has authorized us to
//C- replace the original DjVu(r) Reference Library notice by the following
//C- text (see doc/lizard2002.djvu and doc/lizardtech2007.djvu):
//C-
//C-  ------------------------------------------------------------------
//C- | DjVu (r) Reference Library (v. 3.5)
//C- | Copyright (c) 1999-2001 LizardTech, Inc. All Rights Reserved.
//C- | The DjVu Reference Library is protected by U.S. Pat. No.
//C- | 6,058,214 and patents pending.
//C- |
//C- | This software is subject to, and may be distributed under, the
//C- | GNU General Public License, either Version 2 of the license,
//C- | or (at your option) any later version. The license should have
//C- | accompanied the software or you may obtain a copy of the license
//C- | from the Free Software Foundation at http://www.fsf.org .
//C- |
//C- | The computer code originally released by LizardTech under this
//C- | license and unmodified by other parties is deemed "the LIZARDTECH
//C- | ORIGINAL CODE."  Subject to any third party intellectual property
//C- | claims, LizardTech grants recipient a worldwide, royalty-free, 
//C- | non-exclusive license to make, use, sell, or otherwise dispose of 
//C- | the LIZARDTECH ORIGINAL CODE or of programs derived from the 
//C- | LIZARDTECH ORIGINAL CODE in compliance with the terms of the GNU 
//C- | General Public License.   This grant only confers the right to 
//C- | infringe patent claims underlying the LIZARDTECH ORIGINAL CODE to 
//C- | the extent such infringement is reasonably necessary to enable 
//C- | recipient to make, have made, practice, sell, or otherwise dispose 
//C- | of the LIZARDTECH ORIGINAL CODE (or portions thereof) and not to 
//C- | any greater extent that may be necessary to utilize further 
//C- | modifications or combinations.
//C- |
//C- | The LIZARDTECH ORIGINAL CODE is provided "AS IS" WITHOUT WARRANTY
//C- | OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
//C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
//C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//C- +------------------------------------------------------------------
// 
// $Id: mime_check.cpp,v 1.8 2007/03/25 20:48:23 leonb Exp $
// $Name: HEAD $

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif
#if NEED_GNUG_PRAGMAS
# pragma implementation
#endif

#include "prefs.h"
#include "mime_check.h"
#include "mime_utils.h"
#include "GString.h"
#include "ByteStream.h"
#include "qd_mime_dialog.h"
#include "GURL.h"

#include <stdlib.h>
#include <unistd.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include "debug.h"

class QMimeChecker : public QObject
{
   Q_OBJECT
public slots:
   void	slotCheckMimeTypes(void);
};

void
QMimeChecker::slotCheckMimeTypes(void)
{
   DEBUG_MSG("QMimeChecker::slotCheckMimeTypes\n");
   DjVuPrefs prefs;
   if (prefs.mimeDontCheck) 
     return;
   
   G_TRY 
     {
       if (FixMimeTypes(false) || FixMailCap(false))
         {
           bool do_it = true;
           if (! prefs.mimeDontAsk) 
             {
               do_it = false;
	       QDMimeDialog d(0, "mime_check_dialog", TRUE);
	       if (d.exec() == QDialog::Accepted)
                 {
                   do_it = true;
                   QMessageBox::information(0, "DjVu",
                      tr("Please restart Netscape for the changes to take effect"));
                 }
	       prefs.mimeDontAsk = d.dontAsk();
	       prefs.mimeDontCheck = d.dontCheck();
	       prefs.save();
             }
           if (do_it)
             {
	       fixMime();
             }
         }
     } 
   G_CATCH(exc) 
     {
     } 
   G_ENDCATCH;
}

void
checkMime(void)
{
   DEBUG_MSG("checkMimeTypes\n");
   static QTimer * timer;
   static QMimeChecker * checker;

   DjVuPrefs prefs;
   if (prefs.mimeDontCheck) return;
   if (!timer) timer=new QTimer();
   if (!checker) checker=new QMimeChecker();
   QObject::connect(timer, SIGNAL(timeout(void)), checker, SLOT(slotCheckMimeTypes(void)));
   timer->start(2000, TRUE);
}

void
fixMime(void)
{
   DEBUG_MSG("fixMimeTypes\n");
   FixMimeTypes(true);
   FixMailCap(true);
}

void
fixPlugins(void)
{
  const char *home = getenv("HOME");
  if (! home) return;
  GUTF8String fname = GUTF8String(home) + "/.netscape/plugin-list";
  remove((const char*)fname);
  fname = GUTF8String(home) + "/.netscape/plugin-list.BAK";
  remove((const char*)fname);
}

#include "mime_check_moc.inc"
